@@ -37,7 +37,13 @@
 #  include <png.h>
 # endif
 #else
-#include <setjmp.h>
+# include <setjmp.h>
+# ifdef HAVE_JPEG
+#   ifdef HAVE_UNISTD_H
+#     include <unistd.h>
+#   endif
+#   include <stdio.h>
+# endif
 #endif
 #ifdef HAVE_JPEG
 /* Include file for users of png library. */
@@ -173,25 +179,16 @@ const char *as_image_file_type_names[ASIT_Unknown+1] =
 	"Unknown"
 };
 
- 
-ASImage *
-file2ASImage_extra( const char *file, ASImageImportParams *iparams )
+char *locate_image_file_in_path( const char *file, ASImageImportParams *iparams ) 
 {
 	int 		  filename_len ;
-//	int 		  subimage = -1 ;
 	char 		 *realfilename = NULL, *tmp = NULL ;
 	register int i;
-
-	ASImage *im = NULL;
-	char *g_var ;
 	ASImageImportParams dummy_iparams = {0};
 
 	if( iparams == NULL )
 		iparams = &dummy_iparams ;
 
-	if( (g_var = getenv( "SCREEN_GAMMA" )) != NULL )
-		iparams->gamma = atof(g_var);
-
 	if( file )
 	{
 		filename_len = strlen(file);
@@ -243,27 +240,81 @@ file2ASImage_extra( const char *file, ASImageImportParams *iparams )
 		if( tmp != realfilename && tmp != NULL )
 			free( tmp );
 	}
-	if( realfilename )
+	if( realfilename == file )
+		realfilename = mystrdup(file);
+	return realfilename ;
+}
+ASImage *
+file2ASImage_extra( const char *file, ASImageImportParams *iparams )
+{
+	char *realfilename ;
+	ASImage *im = NULL;
+	ASImageImportParams dummy_iparams = {0};
+	
+	if( iparams == NULL )
+		iparams = &dummy_iparams ;
+
+	realfilename = locate_image_file_in_path( file, iparams ); 
+	
+	if( realfilename != NULL ) 
 	{
 		ASImageFileTypes file_type = check_image_type( realfilename );
+
 		if( file_type == ASIT_Unknown )
 			show_error( "Hmm, I don't seem to know anything about format of the image file \"%s\"\n.\tPlease check the manual", realfilename );
 		else if( as_image_file_loaders[file_type] )
+		{
+			char *g_var = getenv( "SCREEN_GAMMA" );
+			if( g_var != NULL )
+				iparams->gamma = atof(g_var);
 			im = as_image_file_loaders[file_type](realfilename, iparams);
-		else
+		}else
 			show_error( "Support for the format of image file \"%s\" has not been implemented yet.", realfilename );
+		/* returned image must not be tracked by any ImageManager yet !!! */
+		if( im != NULL && im->imageman != NULL ) 
+		{
+			if( im->ref_count == 1 ) 
+			{
+				forget_asimage( im );
+			}else
+			{
+				ASImage *tmp = clone_asimage( im , 0xFFFFFFFF); 
+				if( tmp ) 
+				{
+					release_asimage( im );
+					im = tmp ;
+				}
+			}
+		}
+
 #ifndef NO_DEBUG_OUTPUT
 		if( im != NULL ) 
 			show_progress( "image loaded from \"%s\"", realfilename );
 #endif
-		if( realfilename != file )
-			free( realfilename );
+		free( realfilename );
 	}else
 		show_error( "I'm terribly sorry, but image file \"%s\" is nowhere to be found.", file );
 
 	return im;
 }
 
+void init_asimage_import_params( ASImageImportParams *iparams ) 
+{
+	if( iparams ) 
+	{
+		iparams->flags = 0 ;
+		iparams->width = 0 ;
+		iparams->height = 0 ;
+		iparams->filter = SCL_DO_ALL ;
+		iparams->gamma = 0. ;
+		iparams->gamma_table = NULL ;
+		iparams->compression = 100 ;
+		iparams->format = ASA_ASImage ;
+		iparams->search_path = NULL;
+		iparams->subimage = 0 ;
+	}
+}
+
 ASImage *
 file2ASImage( const char *file, ASFlagType what, double gamma, unsigned int compression, ... )
 {
@@ -272,17 +323,15 @@ file2ASImage( const char *file, ASFlagType what, double gamma, unsigned int comp
 	ASImageImportParams iparams ;
 	va_list       ap;
 
-	iparams.flags = 0 ;
-	iparams.width = 0 ;
-	iparams.height = 0 ;
-	iparams.filter = SCL_DO_ALL ;
+	init_asimage_import_params( &iparams );
 	iparams.gamma = gamma ;
-	iparams.gamma_table = NULL ;
 	iparams.compression = compression ;
-	iparams.format = ASA_ASImage ;
 	iparams.search_path = &(paths[0]);
-	iparams.subimage = 0 ;
-
+#if 0
+	iparams.width = 1024 ; 
+	iparams.height = -1 ; 	
+	iparams.flags |= AS_IMPORT_SCALED_H|AS_IMPORT_SCALED_V ;
+#endif
 	va_start (ap, compression);
 	for( i = 0 ; i < MAX_SEARCH_PATHS ; i++ )
 	{	
@@ -339,19 +388,38 @@ file2pixmap(ASVisual *asv, Window root, const char *realfilename, Pixmap *mask_o
 static ASImage *
 load_image_from_path( const char *file, char **path, double gamma)
 {
-	if( path[2] == NULL )
-		return file2ASImage( file, 0xFFFFFFFF, gamma, 100, path[0], path[1], NULL );
-#if (MAX_SEARCH_PATHS!=8)
-#error "Fixme: please update file2ASImage call to match max number of search paths in ImageManager"
-#else
-	else
-		return file2ASImage( file, 0xFFFFFFFF, gamma, 100, path[0], path[1], path[2],
-			    	       path[3], path[4], path[5], path[6], path[7], NULL );
-#endif
+	ASImageImportParams iparams ;
 
-	return NULL ;
+	init_asimage_import_params( &iparams );
+	iparams.gamma = gamma ;
+	iparams.search_path = path;
+
+	return file2ASImage_extra( file, &iparams );
 }
 
+ASImageFileTypes
+get_asimage_file_type( ASImageManager* imageman, const char *file )
+{
+	ASImageFileTypes file_type = ASIT_Unknown ;
+	if( file )
+	{
+		ASImageImportParams iparams ;
+		char *realfilename ;
+	
+		init_asimage_import_params( &iparams );
+		iparams.search_path = imageman?&(imageman->search_path[0]):NULL;
+		realfilename = locate_image_file_in_path( file, &iparams ); 
+	
+		if( realfilename != NULL ) 
+		{
+			file_type = check_image_type( realfilename );
+			free( realfilename );
+		}
+	}
+	return file_type;
+}
+
+
 ASImage *
 get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsigned int compression )
 {
@@ -370,6 +438,133 @@ get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsign
 	return im;
 }
 
+void 
+calculate_proportions( int src_w, int src_h, int *pdst_w, int *pdst_h ) 
+{
+	int dst_w = pdst_w?*pdst_w:0 ; 
+	int dst_h = pdst_h?*pdst_h:0 ; 
+	
+	if( src_w > 0 && src_w >= src_h && (dst_w > 0 || dst_h <= 0)) 
+		dst_h = (src_h*dst_w)/src_w ; 
+	else if( src_h > 0 ) 
+		dst_w = (src_w*dst_h)/src_h ; 
+
+	if( pdst_w ) *pdst_w = dst_w ; 	
+	if( pdst_h ) *pdst_h = dst_h ; 
+}
+
+void print_asimage_func (ASHashableValue value);
+ASImage *
+get_thumbnail_asimage( ASImageManager* imageman, const char *file, int thumb_width, int thumb_height, ASFlagType flags )
+{
+	ASImage *im = NULL ;
+	
+	if( imageman && file )
+	{
+#define AS_THUMBNAIL_NAME_FORMAT	"%s_scaled_to_%dx%d"
+		char *thumbnail_name = safemalloc( strlen(file)+sizeof(AS_THUMBNAIL_NAME_FORMAT)+32 );
+		ASImage *original_im = query_asimage(imageman, file );
+
+		if( thumb_width <= 0 && thumb_height <= 0 ) 
+		{
+			thumb_width = 48 ;
+			thumb_height = 48 ;
+		}
+
+		if( get_flags(flags, AS_THUMBNAIL_PROPORTIONAL ) ) 
+		{
+			if( original_im != NULL )		
+				calculate_proportions( original_im->width, original_im->height, &thumb_width, &thumb_height );
+		}else
+		{
+			if( thumb_width == 0 ) 
+				thumb_width = thumb_height ; 
+			if( thumb_height == 0 ) 
+				thumb_height = thumb_width ; 
+		}
+
+		if( thumb_width > 0 && thumb_height > 0 ) 
+		{
+			sprintf( thumbnail_name, AS_THUMBNAIL_NAME_FORMAT, file, thumb_width, thumb_height ) ;
+			im = fetch_asimage(imageman, thumbnail_name );
+			if( im == NULL )
+			{
+				if( original_im != NULL ) /* simply scale it down to a thumbnail size */
+				{
+					if( (( original_im->width > thumb_width || original_im->height > thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_REDUCE ) ) ||
+						(( original_im->width < thumb_width || original_im->height < thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_ENLARGE ) ) )
+					{
+						im = scale_asimage( NULL, original_im, thumb_width, thumb_height, ASA_ASImage, 100, ASIMAGE_QUALITY_FAST );
+						if( im != NULL ) 
+							store_asimage( imageman, im, thumbnail_name );
+					}else
+						im = dup_asimage( original_im );
+				}
+			}
+		}
+		
+		if( im == NULL ) 	
+		{
+			ASImage *tmp ; 
+			ASImageImportParams iparams ;
+
+			init_asimage_import_params( &iparams );
+			iparams.gamma = imageman->gamma ;
+			iparams.search_path = &(imageman->search_path[0]);
+			
+			iparams.width = thumb_width ; 
+			iparams.height = thumb_height ; 
+			iparams.flags |= AS_IMPORT_RESIZED|AS_IMPORT_SCALED_BOTH ; 
+			
+			if( get_flags( flags, AS_THUMBNAIL_DONT_ENLARGE ) )
+				iparams.flags |= AS_IMPORT_FAST ; 
+			
+			tmp = file2ASImage_extra( file, &iparams );
+			if( tmp ) 
+			{
+				im = tmp ; 
+				if( tmp->width != thumb_width || tmp->height != thumb_height ) 
+				{
+					if( get_flags(flags, AS_THUMBNAIL_PROPORTIONAL ) ) 
+					{
+						calculate_proportions( tmp->width, tmp->height, &thumb_width, &thumb_height );
+						sprintf( thumbnail_name, AS_THUMBNAIL_NAME_FORMAT, file, thumb_width, thumb_height );
+						if( (im = query_asimage( imageman, thumbnail_name )) == NULL ) 
+							im = tmp ; 
+					}
+					if( im == tmp )
+					{
+						if( (( tmp->width > thumb_width || tmp->height > thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_REDUCE ) ) ||
+							(( tmp->width < thumb_width || tmp->height < thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_ENLARGE ) ) )
+						{
+							im = scale_asimage( NULL, tmp, thumb_width, thumb_height, ASA_ASImage, 100, ASIMAGE_QUALITY_FAST );
+							if( im == NULL ) 
+								im = tmp ;
+						}
+					}
+				}			
+
+				if( im != NULL )
+				{
+					if( im->imageman == NULL )
+						store_asimage( imageman, im, thumbnail_name );
+					else
+						dup_asimage( im );
+				}
+				
+				if( im != tmp ) 
+					destroy_asimage( &tmp );				
+			}
+		
+		}
+								 
+		if( thumbnail_name ) 
+			free( thumbnail_name );
+	}
+	return im;
+}
+
+
 Bool
 reload_asimage_manager( ASImageManager *imman )
 {
@@ -743,12 +938,11 @@ open_image_file( const char *path )
 static ASImageFileTypes
 check_image_type( const char *realfilename )
 {
-	int filename_len = strlen( realfilename );
-#define FILE_HEADER_SIZE	512
-	char head[FILE_HEADER_SIZE+1] ;
-	int bytes_in = 0 ;
-	FILE *fp ;
 	ASImageFileTypes type = ASIT_Unknown ;
+	int filename_len = strlen( realfilename );
+	FILE *fp ;
+#define FILE_HEADER_SIZE	512
+
 	/* lets check if we have compressed xpm file : */
 	if( filename_len > 5 && (mystrncasecmp( realfilename+filename_len-5, ".html", 5 ) == 0 || 
 							 mystrncasecmp( realfilename+filename_len-4, ".htm", 4 ) == 0 ))
@@ -759,8 +953,10 @@ check_image_type( const char *realfilename )
 		type = ASIT_ZCompressedXpm ;
 	else if( (fp = open_image_file( realfilename )) != NULL )
 	{
+		char head[FILE_HEADER_SIZE+1] ;
+		int bytes_in = 0 ;
+		memset(&head[0], 0x00, sizeof(head));
 		bytes_in = fread( &(head[0]), sizeof(char), FILE_HEADER_SIZE, fp );
-		head[FILE_HEADER_SIZE] = '\0' ;
 		DEBUG_OUT("%s: head[0]=0x%2.2X(%d),head[2]=0x%2.2X(%d)\n", realfilename+filename_len-4, head[0], head[0], head[2], head[2] );
 /*		fprintf( stderr, " IMAGE FILE HEADER READS : [%s][%c%c%c%c%c%c%c%c][%s], bytes_in = %d\n", (char*)&(head[0]),
 						head[0], head[1], head[2], head[3], head[4], head[5], head[6], head[7], strstr ((char *)&(head[0]), "XPM"),bytes_in );
@@ -897,6 +1093,8 @@ xpm_file2ASImage( ASXpmFile *xpm_file, unsigned int compression )
 			int bytes_count = im->width*4 ;
 			ASFlagType rgb_flags = ASStorage_RLEDiffCompress|ASStorage_32Bit ;
 			ASFlagType alpha_flags = ASStorage_RLEDiffCompress|ASStorage_32Bit ;
+			int old_storage_block_size = set_asstorage_block_size( NULL, xpm_file->width*xpm_file->height*3/2 );
+
 			if( !xpm_file->full_alpha ) 
 				alpha_flags |= ASStorage_Bitmap ;
 			for( line = 0 ; line < xpm_file->height ; ++line )
@@ -915,6 +1113,7 @@ xpm_file2ASImage( ASXpmFile *xpm_file, unsigned int compression )
 				print_component( xpm_file->scl.blue, 0, xpm_file->width );
 #endif
 			}
+			set_asstorage_block_size( NULL, old_storage_block_size);
 		}
 	return im ;
 }
@@ -1004,6 +1203,7 @@ png2ASImage( const char * path, ASImageImportParams *params )
 	unsigned int  y;
 	size_t		  row_bytes, offset ;
 	static ASImage 	 *im = NULL ;
+	int old_storage_block_size;
 	START_TIME(started);
 
 	im = NULL ;
@@ -1106,6 +1306,8 @@ png2ASImage( const char * path, ASImageImportParams *params )
 
 				/* The easiest way to read the image: */
 				png_read_image (png_ptr, row_pointers);
+				
+				old_storage_block_size = set_asstorage_block_size( NULL, width*height*3/2 );
 				for (y = 0; y < height; y++)
 				{
 					if( do_alpha || !grayscale ) 
@@ -1151,6 +1353,8 @@ png2ASImage( const char * path, ASImageImportParams *params )
 						}
 					}
 				}
+				set_asstorage_block_size( NULL, old_storage_block_size );
+
 				free (row_pointers);
 				if( do_alpha || !grayscale ) 
 					free_scanline(&buf, True);
@@ -1208,6 +1412,7 @@ ASImage *
 jpeg2ASImage( const char * path, ASImageImportParams *params )
 {
 	ASImage *im ;
+	int old_storage_block_size ;
 	/* This struct contains the JPEG decompression parameters and pointers to
 	 * working space (which is allocated as needed by the JPEG library).
 	 */
@@ -1264,6 +1469,51 @@ jpeg2ASImage( const char * path, ASImageImportParams *params )
 	/* Adjust default decompression parameters */
 	cinfo.quantize_colors = FALSE;		       /* we don't want no stinking colormaps ! */
 	cinfo.output_gamma = params->gamma;
+	
+	if( get_flags( params->flags, AS_IMPORT_SCALED_BOTH ) == AS_IMPORT_SCALED_BOTH )
+	{
+		int w = params->width ; 
+		int h = params->height ;
+		int ratio ; 
+
+		if( w == 0 )
+		{
+			if( h == 0 ) 
+			{
+				w = cinfo.image_width ; 
+				h = cinfo.image_height ; 
+			}else
+				w = (cinfo.image_width * h)/cinfo.image_height ;
+		}else if( h == 0 )
+			h = (cinfo.image_height * w)/cinfo.image_width ;
+		
+		ratio = cinfo.image_height/h ; 
+		if( ratio > cinfo.image_width/w )
+			ratio = cinfo.image_width/w ; 
+		
+		cinfo.scale_num = 1 ; 
+		/* only supported values are 1, 2, 4, and 8 */
+		cinfo.scale_denom = 1 ; 
+		if( ratio >= 2 ) 
+		{
+			if( ratio >= 4 ) 
+			{
+				if( ratio >= 8 ) 
+					cinfo.scale_denom = 8 ; 
+				else
+					cinfo.scale_denom = 4 ; 
+			}else
+				cinfo.scale_denom = 2 ; 
+		}
+	}
+	
+	if( get_flags( params->flags, AS_IMPORT_FAST ) )
+	{/* this does not really makes much of a difference */
+		cinfo.do_fancy_upsampling = FALSE ; 
+		cinfo.do_block_smoothing = FALSE ; 
+		cinfo.dct_method = JDCT_IFAST ; 
+	}
+	
 	/* Step 5: Start decompressor */
 	(void)jpeg_start_decompress (&cinfo);
 	LOCAL_DEBUG_OUT("stored image size %dx%d", cinfo.output_width,  cinfo.output_height);
@@ -1284,6 +1534,8 @@ jpeg2ASImage( const char * path, ASImageImportParams *params )
 /*	for( i = 0 ; i < im->width ; i++ )	fprintf( stderr, "%3.3d    ", i );
 	fprintf( stderr, "\n");
  */
+	old_storage_block_size = set_asstorage_block_size( NULL, im->width*im->height*3/2 );
+
  	while ( ++y < (int)cinfo.output_height )
 	{
 		/* jpeg_read_scanlines expects an array of pointers to scanlines.
@@ -1313,6 +1565,7 @@ jpeg2ASImage( const char * path, ASImageImportParams *params )
 		fprintf( stderr, "\n");
  */
 	}
+	set_asstorage_block_size( NULL, old_storage_block_size );
 	if( cinfo.output_components != 1 ) 
 		free_scanline(&buf, True);
 	SHOW_TIME("read",started);
@@ -1863,12 +2116,15 @@ gif2ASImage( const char * path, ASImageImportParams *params )
                 int interlaced = sp->ImageDesc.Interlace;
                 int image_y;
 				CARD8 		 *r = NULL, *g = NULL, *b = NULL, *a = NULL ;
+				int 	old_storage_block_size ;
 				r = safemalloc( width );	   
 				g = safemalloc( width );	   
 				b = safemalloc( width );	   
 				a = safemalloc( width );
 
 				im = create_asimage( width, height, params->compression );
+				old_storage_block_size = set_asstorage_block_size( NULL, im->width*im->height*3/2 );
+
 				for (y = 0; y < height; ++y)
 				{
 					unsigned int x ;
@@ -1896,6 +2152,7 @@ gif2ASImage( const char * path, ASImageImportParams *params )
 					if( do_alpha )
 						im->channels[IC_ALPHA][image_y]  = store_data( NULL, a, im->width, ASStorage_RLEDiffCompress|ASStorage_Bitmap, 0);
 				}
+				set_asstorage_block_size( NULL, old_storage_block_size );
 				free(a);
 				free(b);
 				free(g);
@@ -1988,10 +2245,13 @@ tiff2ASImage( const char * path, ASImageImportParams *params )
 			CARD8 		 *r = NULL, *g = NULL, *b = NULL, *a = NULL ;
 			ASFlagType store_flags = ASStorage_RLEDiffCompress	;
 			int first_row = 0 ;
+			int old_storage_block_size;
 			if( bits == 1 ) 
 				set_flags( store_flags, ASStorage_Bitmap );
 			
 			im = create_asimage( width, height, params->compression );
+			old_storage_block_size = set_asstorage_block_size( NULL, im->width*im->height*3/2 );
+			
 			if( depth == 2 || depth == 4 ) 
 				a = safemalloc( width );
 			r = safemalloc( width );	   
@@ -2042,6 +2302,8 @@ tiff2ASImage( const char * path, ASImageImportParams *params )
 				}
 				first_row += rows_per_strip ;
 		    }
+			set_asstorage_block_size( NULL, old_storage_block_size );
+
 			if( b ) free( b );
 			if( g ) free( g );
 			if( r ) free( r );
@@ -2114,9 +2376,10 @@ svg2ASImage( const char * path, ASImageImportParams *params )
    	GdkPixbuf *pixbuf;
 	int channels ;
 	Bool do_alpha ; 
+	int width = -1, height = -1 ; 
  
 	START_TIME(started);
-
+#if 1
 	/* Damn gtk mess... must init once atleast.. can we just init
 	   several times or do we bork then? */
 	if (gType_inited == 0) 
@@ -2125,7 +2388,13 @@ svg2ASImage( const char * path, ASImageImportParams *params )
 	   gType_inited = 1;
 	}
  
-	if( (pixbuf = rsvg_pixbuf_from_file_at_size(path, -1, -1, NULL)) == NULL )
+ 	if( get_flags( params->flags, AS_IMPORT_SCALED_H ) )
+		width = (params->width <= 0)?((params->height<=0)?-1:params->height):params->width ;
+	
+ 	if( get_flags( params->flags, AS_IMPORT_SCALED_V ) )
+		height = (params->height <= 0)?((params->width <= 0)?-1:params->width):params->height ;
+		
+	if( (pixbuf = rsvg_pixbuf_from_file_at_size( path, width, height, NULL)) == NULL )
 		return NULL ;
 	
 	channels = gdk_pixbuf_get_n_channels(pixbuf) ;
@@ -2137,6 +2406,7 @@ svg2ASImage( const char * path, ASImageImportParams *params )
 		register CARD8 *row = gdk_pixbuf_get_pixels(pixbuf);
 		int y;
 		CARD8 		 *r = NULL, *g = NULL, *b = NULL, *a = NULL ;
+		int old_storage_block_size;
 
 		width = gdk_pixbuf_get_width(pixbuf);
 		height = gdk_pixbuf_get_height(pixbuf);
@@ -2149,6 +2419,7 @@ svg2ASImage( const char * path, ASImageImportParams *params )
 
 
 		im = create_asimage(width, height, params->compression );
+		old_storage_block_size = set_asstorage_block_size( NULL, im->width*im->height*3/2 );
 		for (y = 0; y < height; ++y) 
 		{
 			int x, i = 0 ;
@@ -2173,6 +2444,7 @@ svg2ASImage( const char * path, ASImageImportParams *params )
 					}
 			row += channels*width ;
 		}
+		set_asstorage_block_size( NULL, old_storage_block_size );
 		free(r);
 		free(g);
 		free(b);
@@ -2182,7 +2454,7 @@ svg2ASImage( const char * path, ASImageImportParams *params )
 	
 	if (pixbuf)
 		gdk_pixbuf_unref(pixbuf);
-	
+#endif	
 	SHOW_TIME("image loading",started);
 
 	return im ;
@@ -2383,7 +2655,10 @@ tga2ASImage( const char * path, ASImageImportParams *params )
 		if( success && load_row_func != NULL ) 
 		{	
 			ASImageOutput  *imout ;
+			int old_storage_block_size;
 			im = create_asimage( width, height, params->compression );
+			old_storage_block_size = set_asstorage_block_size( NULL, im->width*im->height*3/2 );
+
 			if((imout = start_image_output( NULL, im, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT)) == NULL )
 			{
         		destroy_asimage( &im );
@@ -2406,6 +2681,8 @@ tga2ASImage( const char * path, ASImageImportParams *params )
 				free_scanline( &buf, True );
 				free( read_buf );
 			}   
+			set_asstorage_block_size( NULL, old_storage_block_size );
+
 		}	  
 	}	 
 	if( im == NULL )
@@ -2432,6 +2709,8 @@ convert_argb2ASImage( ASVisual *asv, int width, int height, ARGB32 *argb, CARD8
 	{	
 		ASScanline    buf;
 		int y ;
+		int old_storage_block_size = set_asstorage_block_size( NULL, im->width*im->height*3 );
+
 		prepare_scanline( im->width, 0, &buf, True );
 		for( y = 0 ; y < height ; ++y ) 
 		{	  
@@ -2448,6 +2727,7 @@ convert_argb2ASImage( ASVisual *asv, int width, int height, ARGB32 *argb, CARD8
 			set_flags( buf.flags, SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE|SCL_DO_ALPHA );
 			imout->output_image_scanline( imout, &buf, 1);
 		}
+		set_asstorage_block_size( NULL, old_storage_block_size );
 		stop_image_output( &imout );
 		free_scanline( &buf, True );
 	}   
